"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildHttpConfig = exports.configRequiresBody = exports.handleResponse = exports.handleResponseData = exports.buildParameters = exports.removeLeadingPathSegmentForPasscode = exports.checkConfiguration = void 0;
const url_1 = require("url");
const lodash_1 = __importDefault(require("lodash"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const constants_1 = require("../../../constants");
const logger_1 = require("../../../logger");
const config_1 = require("../../../config");
const utils_1 = require("../utils");
const ResultHandlerFactory_1 = require("../../../result/ResultHandlerFactory");
const CURLY_OPEN_ENCODED = encodeURIComponent("{");
const CURLY_CLOSED_ENCODED = encodeURIComponent("}");
const checkConfiguration = (config) => {
    if (!config.authorization) {
        throw new Error("no authorization header");
    }
    if (!config.host) {
        throw new Error("no host");
    }
    if (!config.publicfqdn) {
        throw new Error("no publicfqdn");
    }
};
exports.checkConfiguration = checkConfiguration;
const removeLeadingPathSegmentForPasscode = (path) => {
    const authenticationMethod = (0, utils_1.getAuthenticationMethod)();
    if (authenticationMethod === constants_1.AuthenticationMethod.passcode) {
        const segments = path.split("/");
        if (constants_1.SEGMENTS_TO_REMOVE_FOR_PASSCODE_AUTH.includes(segments[1])) {
            segments.splice(1, 1);
            return segments.join("/");
        }
    }
    return path;
};
exports.removeLeadingPathSegmentForPasscode = removeLeadingPathSegmentForPasscode;
const getUrl = (path) => {
    if (path.startsWith("http")) {
        return new url_1.URL(path);
    }
    const pathInt = (0, exports.removeLeadingPathSegmentForPasscode)(path);
    const targetHost = (0, utils_1.getTargetHost)();
    return new url_1.URL(`${targetHost}${pathInt}`);
};
const handleMappingIn = (mapping, url, value, headers, bodyWrapper) => {
    if (mapping.in === "query") {
        url.searchParams.append(mapping.name, value);
    }
    else if (mapping.in === "header") {
        // eslint-disable-next-line no-param-reassign
        headers[mapping.name] = value;
    }
    else if (mapping.in === "path") {
        // eslint-disable-next-line no-param-reassign
        url.pathname = url.pathname.replace(new RegExp(`${CURLY_OPEN_ENCODED}${mapping.name}${CURLY_CLOSED_ENCODED}`, "g"), value);
    }
    else {
        // NOSONAR
        // if (mapping.in === "body") {
        if (!bodyWrapper.body) {
            // eslint-disable-next-line no-param-reassign
            bodyWrapper.body = {};
        }
        lodash_1.default.set(bodyWrapper.body, mapping.name, value);
    }
};
const getValueFromMappping = (mapping, config) => {
    if (mapping.source.type === "value") {
        return mapping.source.value;
    }
    if (
    // mapping.source.type === "option"
    config.options[mapping.source.name] === undefined &&
        mapping.source.dataType === "boolean") {
        return false;
    }
    // mapping.source.type === "option"
    return config.options[mapping.source.name];
};
const handleParameterMapping = (config, url, headers, bodyWrapper) => (mapping) => {
    const value = getValueFromMappping(mapping, config);
    if (value === undefined || value === null) {
        return;
    }
    handleMappingIn(mapping, url, value, headers, bodyWrapper);
};
const buildParameters = (path, parameterMappings) => {
    const config = (0, config_1.get)();
    const headers = {};
    const bodyWrapper = {
        body: config.data,
    };
    const url = getUrl(path);
    parameterMappings?.forEach(handleParameterMapping(config, url, headers, bodyWrapper));
    return {
        url,
        headers,
        body: bodyWrapper.body,
    };
};
exports.buildParameters = buildParameters;
const handleResponseData = async (data) => {
    const config = (0, config_1.get)();
    if (!config.doNotStoreResult) {
        ResultHandlerFactory_1.ResultHandlerFactory.get().setResult(data);
    }
    const { output } = (0, logger_1.get)("handler.fetch");
    const formatted = config.options[constants_1.OPTION_PRETTY.longName]
        ? JSON.stringify(data, null, 2)
        : JSON.stringify(data);
    if (config.options[constants_1.OPTION_OUTPUT.longName]) {
        await fs_extra_1.default.writeFile(config.options[constants_1.OPTION_OUTPUT.longName], formatted);
    }
    else {
        output(formatted);
    }
};
exports.handleResponseData = handleResponseData;
const handleResponse = async (data, headers) => {
    if (headers?.[constants_1.X_CSRF_TOKEN]) {
        (0, config_1.set)({ [constants_1.X_CSRF_TOKEN]: headers[constants_1.X_CSRF_TOKEN] });
    }
    else if (data) {
        await (0, exports.handleResponseData)(data);
    }
};
exports.handleResponse = handleResponse;
const configRequiresBody = (method) => ["POST", "PUT", "PATCH"].includes(method.toUpperCase());
exports.configRequiresBody = configRequiresBody;
const buildHttpConfig = (method, path, parameterMappings) => {
    const config = (0, config_1.get)();
    const { url, body, headers } = (0, exports.buildParameters)(path, parameterMappings);
    const httpConfig = {
        url: url.toString(),
        method,
        data: (0, exports.configRequiresBody)(method) ? body : undefined,
        headers: {
            ...config.authorization,
            ...headers,
            publicfqdn: config.publicfqdn,
        },
    };
    if ((0, utils_1.getAuthenticationMethod)() === constants_1.AuthenticationMethod.oauth) {
        delete httpConfig.headers.publicfqdn;
    }
    return httpConfig;
};
exports.buildHttpConfig = buildHttpConfig;
